@@ -21,7 +21,7 @@ from enum import Enum
 from typing import Dict, List, NamedTuple, Optional, Set, Tuple, Union
 import unittest
 
-from typedload import datadumper, dataloader, dump, load
+from typedload import datadumper, dump, load
 
 
 class EnumA(Enum):
@@ -90,7 +90,7 @@ class TestBasicDump(unittest.TestCase):
             assert dumper.dump(True) == True
 
 
-class TestHandlers(unittest.TestCase):
+class TestHandlersDumper(unittest.TestCase):
 
     def test_custom_handler(self):
         class Q:
@@ -111,3 +111,17 @@ class TestHandlers(unittest.TestCase):
             dumper.dump(1)
         dumper.raiseconditionerrors = False
         assert dumper.dump(1) == 1
+
+    def test_replace_handler(self):
+        dumper = datadumper.Dumper()
+        index = dumper.index([])
+        assert dumper.dump([11]) == [11]
+        dumper.handlers[index] = (dumper.handlers[index][0], lambda *args: 3)
+        assert dumper.dump([11]) == 3
+
+
+class TestDumper(unittest.TestCase):
+
+    def test_kwargs(self):
+        with self.assertRaises(ValueError):
+            dump(1, handlers=[])
