@@ -22,6 +22,8 @@
 from enum import Enum
 from typing import *
 
+from .exceptions import TypedloadValueError
+
 
 __all__ = [
     'Dumper',
@@ -117,7 +119,7 @@ class Dumper:
                 match = False
             if match:
                 return i
-        raise ValueError('Unable to dump %s' % value)
+        raise TypedloadValueError('Unable to dump %s' % value, value=value)
 
     def dump(self, value: Any) -> Any:
         index = self.index(value)
@@ -135,7 +137,7 @@ def _namedtupledump(l, value):
 
 
 def _dataclassdump(l, value):
-    import dataclasses  # type: ignore
+    import dataclasses
     fields = set(value.__dataclass_fields__.keys())
     field_defaults = {k: v.default for k,v in value.__dataclass_fields__.items() if not isinstance (v.default, dataclasses._MISSING_TYPE)}
     return {
