@@ -22,6 +22,8 @@
 from enum import Enum
 from typing import *
 
+from .exceptions import *
+
 
 __all__ = [
     'Loader',
@@ -31,6 +33,12 @@ __all__ = [
 NONETYPE = type(None)  # type: Type[Any]
 T = TypeVar('T')
 
+try:
+    # Since 3.7
+    from typing import ForwardRef  # type: ignore
+except ImportError:
+    from typing import _ForwardRef as ForwardRef  # type: ignore
+
 
 def _issubclass(t1, t2) -> bool:
     """
@@ -66,7 +74,7 @@ class Loader:
         exceptions if there are fields in the data that
         are not being used by the type.
 
-    basiccast: When enabled, instead of trying to perform
+    basiccast: When disabled, instead of trying to perform
         casts, exceptions will be raised.
         Since many json seem to encode numbers as strings,
         to avoid extra complications this functionality is
@@ -96,6 +104,26 @@ class Loader:
         In most cases, it is sufficient to append new elements
         at the end, to handle more types.
 
+    frefs: Dictionary to resolve ForwardRef.
+        Something like
+        class Node(NamedTuple):
+            next: Optional['Node']
+
+        requires a ForwardRef (also in python3.7), which means that the type
+        is stored as string and must be resolved at runtime.
+
+        This dictionary contains the names of the types as keys, and the
+        actual types as values.
+
+        A loader object by default starts with an empty dictionary and
+        fills it with the types it encounters, but it is possible to
+        manually add more types to the dictionary.
+
+        Setting this to None disables any support for ForwardRef.
+
+        Reusing the same loader object on unrelated types might cause
+        failures, if the types are different but use the same names.
+
     These parameters can be set as named arguments in the constructor
     or they can be set later on.
 
@@ -113,6 +141,7 @@ class Loader:
         * Tuple[TypeA, TypeB, TypeC]
         * Set[SomeType]
         * Union[TypeA, TypeB]
+        * ForwardRef
 
     Using unions is complicated. If the types in the union are too
     similar to each other, it is easy to obtain an unexpected type.
@@ -134,6 +163,9 @@ class Loader:
         # Raise errors if the condition fails
         self.raiseconditionerrors = True
 
+        # Forward refs dictionary
+        self.frefs = {}  # type: Optional[Dict[str, Type]]
+
         if HAS_UNIONSUBCLASS:
             # Old python
             union_check = lambda type_: _issubclass(type_, Union)
@@ -160,6 +192,7 @@ class Loader:
             (lambda type_: getattr(type_, '__origin__', None) in {set, Set}, _setload),
             (lambda type_: _issubclass(type_, tuple) and set(dir(type_)).issuperset({'_field_types', '_fields'}), _namedtupleload),
             (lambda type_: '__dataclass_fields__' in dir(type_), _namedtupleload),
+            (lambda type_: type(type_) == ForwardRef, _forwardrefload),
         ]  # type: List[Tuple[Callable[[Type[T]], bool], Callable[['Loader', Any, Type[T]], T]]]
 
         for k, v in kwargs.items():
@@ -183,7 +216,7 @@ class Loader:
                 return i
         raise ValueError('No matching condition found')
 
-    def load(self, value: Any, type_: Type[T]) -> T:
+    def load(self, value: Any, type_: Type[T], *, annotation: Optional[Annotation] = None) -> T:
         """
         Loads value into the typed data structure.
 
@@ -193,10 +226,45 @@ class Loader:
         try:
             index = self.index(type_)
         except ValueError:
-            raise TypeError('Cannot deal with value %s of type %s' % (value, type_))
+            raise TypedloadTypeError(
+                'Cannot deal with value of type %s' % type_,
+                value=value,
+                type_=type_
+            )
+
+        # Add type to known types, to resolve ForwardRef later on
+        if self.frefs is not None and hasattr(type_, '__name__'):
+            tname = type_.__name__
+            if tname not in self.frefs:
+                self.frefs[tname] = type_
 
         func = self.handlers[index][1]
-        return func(self, value, type_)
+        try:
+            return func(self, value, type_)
+        except Exception as e:
+            assert isinstance(e, TypedloadException)
+            e.trace.insert(0, TraceItem(value, type_, annotation))
+            raise e
+
+
+def _forwardrefload(l: Loader, value: Any, type_: type) -> Any:
+    """
+    This resolves a ForwardRef.
+
+    It just looks up the type in the dictionary of known types
+    and loads the value using that.
+    """
+    if l.frefs is None:
+        raise TypedloadException('ForwardRef resolving is disabled for the loader', value=value, type_=type_)
+    tname = type_.__forward_arg__  # type: ignore
+    t = l.frefs.get(tname)
+    if t is None:
+        raise TypedloadValueError(
+            "ForwardRef '%s' unknown" % tname,
+            value=value,
+            type_=type_
+        )
+    return l.load(value, t, annotation=Annotation(AnnotationType.FORWARDREF, tname))
 
 
 def _basicload(l: Loader, value: Any, type_: type) -> Any:
@@ -211,9 +279,16 @@ def _basicload(l: Loader, value: Any, type_: type) -> Any:
 
     if type(value) != type_:
         if l.basiccast:
-            return type_(value)
+            try:
+                return type_(value)
+            except ValueError as e:
+                raise TypedloadValueError(str(e), value=value, type_=type_)
+            except TypeError as e:
+                raise TypedloadTypeError(str(e), value=value, type_=type_)
+            except Exception as e:
+                raise TypedloadException(str(e), value=value, type_=type_)
         else:
-            raise ValueError('%s is not of type %s' % (value, type_))
+            raise TypedloadValueError('Not of type %s' % type_, value=value, type_=type_)
     return value
 
 
@@ -222,7 +297,7 @@ def _listload(l: Loader, value, type_) -> List:
     This loads into something like List[int]
     """
     t = type_.__args__[0]
-    return [l.load(v, t) for v in value]
+    return [l.load(v, t, annotation=Annotation(AnnotationType.INDEX, i)) for i, v in enumerate(value)]
 
 
 def _dictload(l: Loader, value, type_) -> Dict:
@@ -232,7 +307,13 @@ def _dictload(l: Loader, value, type_) -> Dict:
     Recursively loads both keys and values.
     """
     key_type, value_type = type_.__args__
-    return {l.load(k, key_type): l.load(v, value_type) for k, v in value.items()}
+    try:
+        return {
+            l.load(k, key_type, annotation=Annotation(AnnotationType.KEY, k)): l.load(v, value_type, annotation=Annotation(AnnotationType.VALUE, v))
+            for k, v in value.items()}
+    except AttributeError as e:
+        raise TypedloadAttributeError(str(e), type_=type_, value=value)
+
 
 
 def _setload(l: Loader, value, type_) -> Set:
@@ -252,11 +333,11 @@ def _tupleload(l: Loader, value, type_) -> Tuple:
     else:
         args = type_.__tuple_params__
     if l.failonextra and len(value) > len(args):
-        raise ValueError('Value %s is too long for type %s' % (value, type_))
+        raise TypedloadValueError('Value is too long for type %s' % type_, value=value, type_=type_)
     elif len(value) < len(args):
-        raise ValueError('Value %s is too short for type %s' % (value, type_))
+        raise TypedloadValueError('Value is too short for type %s' % type_, value=value, type_=type_)
 
-    return tuple(l.load(v, t) for v, t in zip(value, args))
+    return tuple(l.load(v, t, annotation=Annotation(AnnotationType.INDEX, i)) for i, (v, t) in enumerate(zip(value, args)))
 
 
 def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
@@ -269,34 +350,44 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
         type_hints = type_._field_types
     else:
         #dataclass
-        import dataclasses  # type: ignore
+        import dataclasses
         fields = set(type_.__dataclass_fields__.keys())
         optional_fields ={k for k,v in type_.__dataclass_fields__.items() if not isinstance (getattr(v, 'default', dataclasses._MISSING_TYPE()), dataclasses._MISSING_TYPE)}
         type_hints = {k: v.type for k,v in type_.__dataclass_fields__.items()}
     necessary_fields = fields.difference(optional_fields)
-    vfields = set(value.keys())
+    try:
+        vfields = set(value.keys())
+    except AttributeError as e:
+        raise TypedloadAttributeError(str(e), value=value, type_=type_)
 
     if necessary_fields.intersection(vfields) != necessary_fields:
-        raise ValueError(
-            'Value %s does not contain fields: %s which are necessary for type %s' % (
-                value,
+        raise TypedloadValueError(
+            'Value does not contain fields: %s which are necessary for type %s' % (
                 necessary_fields.difference(vfields),
                 type_
-            )
+            ),
+            value=value,
+            type_=type_,
         )
 
     fieldsdiff = vfields.difference(fields)
     if l.failonextra and len(fieldsdiff):
         extra = ', '.join(fieldsdiff)
-        raise ValueError(
-            'Dictionary %s has unrecognized fields: %s and cannot be loaded into %s' % (value, extra, type_)
+        raise TypedloadValueError(
+            'Dictionary has unrecognized fields: %s and cannot be loaded into %s' % (extra, type_),
+            value=value,
+            type_=type_,
         )
 
     params = {}
     for k, v in value.items():
         if k not in fields:
             continue
-        params[k] = l.load(v, type_hints[k])
+        params[k] = l.load(
+            v,
+            type_hints[k],
+            annotation=Annotation(AnnotationType.FIELD, k),
+        )
     return type_(**params)
 
 
@@ -316,7 +407,7 @@ def _unionload(l: Loader, value, type_) -> Any:
     elif hasattr(type_, '__union_params__'):
         args = type_.__union_params__
     else:
-        raise AttributeError('The typing API for this Python version is unknown')
+        raise TypedloadAttributeError('The typing API for this Python version is unknown')
 
     # Do not convert basic types, if possible
     if type(value) in set(args).intersection(l.basictypes):
@@ -327,15 +418,14 @@ def _unionload(l: Loader, value, type_) -> Any:
     # Try all types
     for t in args:
         try:
-            return l.load(value, t)
+            return l.load(value, t, annotation=Annotation(AnnotationType.UNION, t))
         except Exception as e:
-            exceptions.append(str(e))
-    raise ValueError(
-        'Value %s could not be loaded into %s\n\nConversion exceptions were:\n%s' % (
-            value,
-            type_,
-            '\n'.join(exceptions)
-        )
+            exceptions.append(e)
+    raise TypedloadValueError(
+        'Value could not be loaded into %s' % type_,
+            value=value,
+            type_=type_,
+            exceptions=exceptions
     )
 
 
@@ -363,7 +453,11 @@ def _enumload(l: Loader, value, type_) -> Enum:
             return type_(l.load(value, t))
         except:
             pass
-    raise ValueError('Value %s could not be loaded into %s' % (value, type_))
+    raise TypedloadValueError(
+        'Value could not be loaded into %s' % type_,
+        value=value,
+        type_=type_
+    )
 
 
 def _noneload(l: Loader, value, type_) -> None:
@@ -373,4 +467,4 @@ def _noneload(l: Loader, value, type_) -> None:
     """
     if value is None:
         return None
-    raise ValueError('%s is not None' % value)
+    raise TypedloadValueError('Not None', value=value, type_=type_)
