@@ -1,5 +1,17 @@
 # typedload
 # Module to load data into data structures from the "attr" module
+#
+# Name mangling is supported by having a 'name' attribute in the metadata
+#
+# @attr.s
+# class Example:
+#     attribute = attr.ib(type=int, metadata={'name': 'att.rib.ute:name'}
+#
+# The dictionary key for 'attribute' will be 'att.rib.ute:name'.
+#
+# This is very useful for keys that use invalid or reserved characters that
+# can't be used in variable names.
+# Another common application is to convert camelCase into not_camel_case.
 
 # Copyright (C) 2018 Salvo "LtWorf" Tomaselli
 #
@@ -33,7 +45,8 @@ def _attrdump(d, value) -> Dict[str, Any]:
         if not attr.repr:
             continue
         if not (d.hidedefault and attrval == attr.default):
-            r[attr.name] = d.dump(attrval)
+            name = attr.metadata.get('name', attr.name)
+            r[name] = d.dump(attrval)
     return r
 
 
